import {createApp, h} from 'vue'
import {PluginUseManager, SCOPE} from './plugin-use'

export class Application {
    constructor() {
        this.instance = null
        this.pluginUseManager = null

        this.clientIps = []
        this.maintenanceMode = null
        this.limitationMode = null
    }

    maintenance(maintenanceMode) {
        if (typeof maintenanceMode !== 'undefined') {
            this.maintenanceMode = maintenanceMode
        }
        return this.maintenanceMode
    }

    limitation(limitationMode) {
        if (typeof limitationMode !== 'undefined') {
            this.limitationMode = limitationMode
        }
        return this.limitationMode
    }

    ips(clientIps) {
        if (typeof clientIps !== 'undefined') {
            this.clientIps = clientIps
        }
        return this.clientIps
    }

    get() {
        return new Promise((resolve, reject) => {
            const tryInstance = (tried = 1, maxTried = 100) => {
                if (this.instance) {
                    resolve(this.instance)
                    return
                }

                if (tried === maxTried) {
                    reject()
                    return
                }

                setTimeout(() => tryInstance(++tried), 200)
            }

            tryInstance()
        })
    }

    use(pluginUses = {}) {
        Object.keys(pluginUses).forEach(name => this.pluginUseManager.use(name, pluginUses[name]))
        return this
    }

    register(handle) {
        let register = false
        if (!register) {
            register = true
            handle(this)
        }
        return this
    }

    pluginsReady() {
        return this.pluginUseManager.resetAttaching()
    }

    initialize(appComponent) {
        this.instance = createApp({
            render: () => h(appComponent),
        })
        this.pluginUseManager = new PluginUseManager(this.instance)
        return this
    }

    createDefault(appComponent, appElement = '#app') {
        this.instance.mount(appElement)
    }

    createFailed(appFailedComponent, appElement = '#app') {
        this.pluginUseManager.resetAttaching()
        this.pluginUseManager.ready(SCOPE.failed)
            .then(() => this.createDefault(appFailedComponent, appElement))
            .catch(() => this.createDefault(appFailedComponent, appElement))
    }

    create(appComponent, appFailedComponent, appElement = '#app') {
        this.pluginUseManager.resetAttaching()
        this.pluginUseManager.ready(SCOPE.default)
            .then(() => this.createDefault(appComponent, appElement))
            .catch(() => this.createFailed(appFailedComponent, appElement))
    }
}
